<?php

/**
 * Plugin Name: Video Embed AI
 * Description: Provides a button in the settings to automatically embed a YouTube video related to the post's title into posts, using the user's YouTube API key.
 * Version: 1.0
 * Author: Johnny B Good
 * Author URI: https://botsy.ai
 */
// Exit if accessed directly

if ( !function_exists( 'bvea_fs' ) ) {
    // Create a helper function for easy SDK access.
    function bvea_fs()
    {
        global  $bvea_fs ;
        
        if ( !isset( $bvea_fs ) ) {
            // Include Freemius SDK.
            require_once dirname( __FILE__ ) . '/freemius/start.php';
            $bvea_fs = fs_dynamic_init( array(
                'id'             => '14148',
                'slug'           => 'botsy-video-embed-ai',
                'type'           => 'plugin',
                'public_key'     => 'pk_0846aea7bbd57efbccfade5890561',
                'is_premium'     => false,
                'premium_suffix' => 'Professional',
                'has_addons'     => false,
                'has_paid_plans' => true,
                'menu'           => array(
                'slug'    => 'tb-youtube-embed-admin',
                'support' => false,
                'parent'  => array(
                'slug' => 'options-general.php',
            ),
            ),
                'is_live'        => true,
            ) );
        }
        
        return $bvea_fs;
    }
    
    // Init Freemius.
    bvea_fs();
    // Signal that SDK was initiated.
    do_action( 'bvea_fs_loaded' );
}

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
class TitleBasedYouTubeEmbed
{
    private  $youtube_api_key ;
    private  $video_count_option = 'tb_youtube_daily_video_count' ;
    private  $reset_time_option = 'tb_youtube_reset_time' ;
    private  $embedded_posts_option = 'tb_youtube_embedded_posts' ;
    public function __construct()
    {
        register_activation_hook( __FILE__, array( $this, 'activate_plugin' ) );
        add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
        add_action( 'admin_init', array( $this, 'page_init' ) );
        add_action( 'admin_post_update_posts', array( $this, 'handle_update_posts' ) );
        // Check for and reset the video count daily
        $last_reset = get_option( $this->reset_time_option, false );
        $today = date( 'Y-m-d' );
        
        if ( $last_reset !== $today ) {
            update_option( $this->video_count_option, 0 );
            update_option( $this->reset_time_option, $today );
        }
    
    }
    
    public function activate_plugin()
    {
        update_option( $this->video_count_option, 0 );
        update_option( $this->reset_time_option, current_time( 'timestamp' ) );
        // Store the current timestamp
    }
    
    private function reset_daily_count()
    {
        $last_reset = get_option( $this->reset_time_option, 0 );
        $current_time = current_time( 'timestamp' );
        // Check if the current date is greater than the last reset date
        
        if ( date( 'Y-m-d', $current_time ) > date( 'Y-m-d', $last_reset ) ) {
            update_option( $this->video_count_option, 0 );
            // Reset the count
            update_option( $this->reset_time_option, $current_time );
            // Update the reset time
        }
    
    }
    
    public function add_plugin_page()
    {
        add_options_page(
            'Settings Admin',
            'Video Embed AI',
            'manage_options',
            'tb-youtube-embed-admin',
            array( $this, 'create_admin_page' )
        );
    }
    
    public function create_admin_page()
    {
        if ( !current_user_can( 'manage_options' ) ) {
            return;
        }
        $notice = get_transient( 'tb_youtube_embed_admin_notice' );
        
        if ( $notice ) {
            echo  "<div class='notice notice-success is-dismissible'><p>{$notice}</p></div>" ;
            delete_transient( 'tb_youtube_embed_admin_notice' );
        }
        
        $counts = $this->get_post_counts();
        $remaining_videos = 100 - get_option( $this->video_count_option, 0 );
        ?>
        <div class="wrap">
            <h2>Video Embed AI</h2>
            <p>Total Posts: <?php 
        echo  $counts['total'] ;
        ?></p>
            <p>Posts with Videos: <?php 
        echo  $counts['with_videos'] ;
        ?></p>
            <p>Posts without Videos: <?php 
        echo  $counts['without_videos'] ;
        ?></p>
            <p>Remaining Video Embeds Today: <?php 
        echo  $remaining_videos ;
        ?></p>

            <form method="post" action="options.php">
                <?php 
        settings_fields( 'tb_youtube_embed_option_group' );
        do_settings_sections( 'tb-youtube-embed-admin' );
        submit_button();
        ?>
            </form>
            <form method="post" action="<?php 
        echo  esc_url( admin_url( 'admin-post.php' ) ) ;
        ?>">
                <input type="hidden" name="action" value="update_posts" />
                <label for="video_count">Select Number of Videos to Process: </label>
                <select name="video_count" id="video_count">
                	<option value="2">2</option>
                    <?php 
        if ( bvea_fs()->is_plan( 'professional', true ) ) {
            ?>
                    <option value="10">10</option>
                    <option value="25">25</option>
                    <option value="50">50</option>
                    <option value="100">100</option>
                    <?php 
        }
        ?>

                </select>
                
                <?php 
        wp_nonce_field( 'tb_youtube_embed_update_action', 'tb_youtube_embed_nonce' );
        submit_button( 'Update Posts with YouTube Videos' );
        ?>
                                <!-- Add your text or HTML here -->
                <p>*** Daily limit of 100 videos is due to limits set by Google. ***</p>
                <p>To get your own API visit <a href="https://console.cloud.google.com/" target="_blank" rel="noopener noreferrer">Google Cloud Console</a></p>
                <p>For a step by step guide on how to get a Youtube API code, <a href="https://botsy.ai/video-embed-ai-tutorial" target="_blank" rel="noopener noreferrer">Click Here</a></p>
            </form>

    <div class="wrap">
        <!-- ... -->
        <p>Remaining Video Embeds Today: <?php 
        echo  $remaining_videos ;
        ?></p>

        <!-- ... -->
        </div>
        <?php 
    }
    
    public function get_post_counts()
    {
        global  $wpdb ;
        $total = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_status = 'publish' AND post_type = 'post'" );
        $with_videos = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_status = 'publish' AND post_type = 'post' AND post_content LIKE '%src=\"https://www.youtube.com/embed/%'" );
        $without_videos = $total - $with_videos;
        return compact( 'total', 'with_videos', 'without_videos' );
    }
    
    public function page_init()
    {
        register_setting( 'tb_youtube_embed_option_group', 'tb_youtube_embed_options', array( $this, 'sanitize' ) );
        add_settings_section(
            'setting_section_id',
            'Settings',
            array( $this, 'print_section_info' ),
            'tb-youtube-embed-admin'
        );
        add_settings_field(
            'youtube_api_key',
            'YouTube API Key',
            array( $this, 'youtube_api_key_callback' ),
            'tb-youtube-embed-admin',
            'setting_section_id'
        );
    }
    
    public function sanitize( $input )
    {
        $new_input = array();
        if ( isset( $input['youtube_api_key'] ) ) {
            $new_input['youtube_api_key'] = sanitize_text_field( $input['youtube_api_key'] );
        }
        return $new_input;
    }
    
    public function print_section_info()
    {
        print 'Enter your YouTube API Key:';
    }
    
    public function youtube_api_key_callback()
    {
        $options = get_option( 'tb_youtube_embed_options' );
        ?>
        <input type="text" id="youtube_api_key" name="tb_youtube_embed_options[youtube_api_key]" value="<?php 
        echo  esc_attr( $options['youtube_api_key'] ) ;
        ?>" size="50" />
        <?php 
    }
    
    public function handle_update_posts()
    {
        
        if ( !isset( $_POST['tb_youtube_embed_nonce'] ) || !wp_verify_nonce( $_POST['tb_youtube_embed_nonce'], 'tb_youtube_embed_update_action' ) ) {
            set_transient( 'tb_youtube_embed_admin_notice', 'Oops, something happened: Nonce verification failed.', 10 );
            wp_redirect( add_query_arg( 'page', 'tb-youtube-embed-admin', admin_url( 'options-general.php' ) ) );
            exit;
        }
        
        if ( !current_user_can( 'manage_options' ) ) {
            wp_die( 'You do not have sufficient permissions to access this page.' );
        }
        $this->youtube_api_key = get_option( 'tb_youtube_embed_options' )['youtube_api_key'];
        $video_count = get_option( $this->video_count_option, 0 );
        
        if ( $video_count >= 100 ) {
            set_transient( 'tb_youtube_embed_admin_notice', 'Limited to 3-100 videos a day, please try again tomorrow.', 10 );
            wp_redirect( add_query_arg( 'page', 'tb-youtube-embed-admin', admin_url( 'options-general.php' ) ) );
            exit;
        }
        
        // Get the selected video count from the form submission
        $selected_video_count = ( isset( $_POST['video_count'] ) ? intval( $_POST['video_count'] ) : 100 );
        // Ensure the selected video count is one of the allowed values
        if ( !in_array( $selected_video_count, array(
            2,
            10,
            25,
            50,
            100
        ), true ) ) {
            $selected_video_count = 100;
        }
        $update_results = $this->update_recent_posts( $video_count, $selected_video_count );
        // Update the video count option with the total number of videos embedded
        update_option( $this->video_count_option, $video_count + count( $update_results['updated_posts'] ) );
        // Updated line
        $messages = array();
        
        if ( !empty($update_results['updated_posts']) ) {
            // Generate links for updated posts
            $linked_titles = array();
            foreach ( $update_results['updated_posts'] as $title ) {
                $post_id = get_page_by_title( $title, OBJECT, 'post' )->ID;
                // Get the post ID by title
                $permalink = get_permalink( $post_id );
                // Get permalink by ID
                $linked_titles[] = '<a href="' . esc_url( $permalink ) . '" target="_blank">' . $title . '</a>';
            }
            $messages[] = 'Added new videos to:<ol><li>' . implode( '</li><li>', $linked_titles ) . '</li></ol>';
        }
        
        if ( !empty($update_results['already_has_video']) ) {
            // Here, you might also want to generate clickable titles for posts that already have videos.
            // The process would be similar to the one above.
            $messages[] = 'Posts already having videos:<ol><li>' . implode( '</li><li>', $update_results['already_has_video'] ) . '</li></ol>';
        }
        if ( !empty($update_results['errors']) ) {
            $messages[] = 'Errors: ' . implode( ', ', $update_results['errors'] );
        }
        set_transient( 'tb_youtube_embed_admin_notice', implode( ' ', $messages ), 10 );
        wp_redirect( add_query_arg( 'page', 'tb-youtube-embed-admin', admin_url( 'options-general.php' ) ) );
        exit;
    }
    
    private function update_recent_posts( $video_count, $selected_video_count )
    {
        global  $wpdb ;
        // Get the posts without videos up to the selected video count
        $posts = $wpdb->get_col( $wpdb->prepare( "SELECT ID FROM {$wpdb->posts} WHERE post_status = 'publish' AND post_type = 'post' AND post_content NOT LIKE '%%src=\"https://www.youtube.com/embed/%%' ORDER BY post_date DESC LIMIT %d", $selected_video_count ) );
        $updated_posts = array();
        $already_has_video = array();
        $errors = array();
        foreach ( $posts as $post_id ) {
            $post = get_post( $post_id );
            
            if ( strpos( $post->post_content, 'src="https://www.youtube.com/embed/' ) !== false ) {
                $already_has_video[] = $post->post_title;
                continue;
                // Skip posts that already have videos
            }
            
            $video_id = $this->get_youtube_id_by_title( $post->post_title );
            
            if ( !$video_id ) {
                $errors[] = "Failed to find video for post {$post_id}";
                continue;
            }
            
            // Prepare the video embed code, wrapped in a div for centering
            $video_embed_code = sprintf( '<div style="text-align:center; margin-bottom:30px;"><iframe width="560" height="315" src="https://www.youtube.com/embed/%s" frameborder="0" allowfullscreen></iframe></div>', $video_id );
            // Find the position of the first <h2> tag
            $h2pos = strpos( $post->post_content, '<h2>' );
            
            if ( $h2pos !== false ) {
                // If an <h2> tag is found, insert the video embed code before it
                $new_content = substr_replace(
                    $post->post_content,
                    $video_embed_code,
                    $h2pos,
                    0
                );
            } else {
                // If no <h2> tag is found, append the video at the end
                $new_content = $post->post_content . "\n\n" . $video_embed_code;
            }
            
            // Prepare the updated post data
            $updated_post_data = array(
                'ID'           => $post_id,
                'post_content' => $new_content,
            );
            // Update the post
            
            if ( wp_update_post( $updated_post_data ) ) {
                $updated_posts[] = $post->post_title;
            } else {
                $errors[] = "Failed to update post {$post_id}";
            }
            
            
            if ( count( $updated_posts ) >= $selected_video_count ) {
                break;
                // Stop when the selected video count is reached
            }
        
        }
        return compact( 'updated_posts', 'already_has_video', 'errors' );
    }
    
    private function get_recent_posts( $limit )
    {
        global  $wpdb ;
        return $wpdb->get_col( $wpdb->prepare( "SELECT ID FROM {$wpdb->posts} WHERE post_status = 'publish' AND post_type = 'post' ORDER BY post_date DESC LIMIT %d", $limit ) );
    }
    
    private function get_youtube_id_by_title( $title )
    {
        $api_url = 'https://www.googleapis.com/youtube/v3/search?part=snippet&q=' . urlencode( $title ) . '&key=' . $this->youtube_api_key . '&type=video&maxResults=1';
        $response = wp_remote_get( $api_url );
        
        if ( is_wp_error( $response ) ) {
            error_log( 'Request to YouTube API failed for title "' . $title . '" with error: ' . $response->get_error_message() );
            return false;
        }
        
        $body = wp_remote_retrieve_body( $response );
        $json = json_decode( $body );
        
        if ( !is_array( $json->items ) || empty($json->items) ) {
            error_log( 'No video found for title "' . $title . '" or there was an error decoding the YouTube response.' );
            return false;
        }
        
        return $json->items[0]->id->videoId;
    }

}
// Instantiate the plugin class
$tb_youtube_embed = new TitleBasedYouTubeEmbed();